{**********************************************************************}
{ PROGRAMM COMFIND                                                     }
{                                                                      }
{ Funktion:     Suchen der Basisadresse einer seriellen Schnittstelle  }
{               durch Verknpfung folgender Bedingungen:               }
{           1.  Der gesetzten Bits 5 und 6 im LSR-Register bei nicht   }
{               "sendendem"  UART.                                     }
{           2.  Den nichtgesetzten Bits 3-7 im IIR-Register            }
{           3.  Beruecksichtigung der Tatsache das eine gueltige       }
{               Basisadresse nur im Bereich XX0 oder XX8 liegen kann.  }
{                                                                      }
{               Als Zusatzfunktion wird der Status der IRQ-Leitung     }
{               ausgegeben um Konfigurationsfehler zu erkennen.        }
{                                                                      }
{ Das Programm wurde von mir zum Auffinden der Basisadressen von       }
{ undokumentierten COM-Ports COM3,COM4 erstellt.                       }
{                                                                      }
{ V 1.3 25.03.93 DL5FBD Gerd Michael Ritter                            } 
{**********************************************************************}

PROGRAM COMFIND;


FUNCTION HEXBYTE(DEZB:Byte):STRING;   {wandelt BYTE 0-255 in HEXwertstring}

VAR LOB,HIB :BYTE;
    HIS     :STRING[2];

BEGIN
  HIB:=(DEZB AND $F0) DIV 16;
  LOB:=(DEZB AND $0F);
  HIS:='';
  IF HIB<10 THEN HIS:=HIS+CHR(HIB+$30) ELSE HIS:=HIS+CHR(HIB+$41-10);
  IF LOB<10 THEN HIS:=HIS+CHR(LOB+$30) ELSE HIS:=HIS+CHR(LOB+$41-10);
  HEXBYTE:=HIS;
END;


FUNCTION HEXWORD(DWORD:WORD):STRING;   {wandelt WORD 0-65535 in HEXwertstring}

VAR HIS:   STRING [4];

BEGIN
  HIS:=HEXBYTE(HI(DWORD));
  HIS:=HIS+HEXBYTE(LO(DWORD));
  HEXWORD:=HIS;
END;


CONST Version ='1.3'; {Programmversion}
      LSRMSK  =$60;   {Bitmaske fuer Line Statusregister}
      LSROFF  =5;     {Offset Line Status Register}
      IIROFF  =3;     {Offset Interrupt Identification Register}
      FCROff  =3;     {Offset FIFO Control Register des 16550}
      MCROff  =4;     {Offset Modem Status Register}
      OUT2    =8;     {Bitmaske OUT2=IRQ-Freigabe}
      FIFOON  =1;     {FIFO des 16550 ein, Triggerschwelle 1 Zeichen}
      IOMIN   =$100;  {untere Grenze IO-Bereich (EA-Karten)}
      IOMAX   =$3ff;  {obere  Grenze IO-Bereich (EA-Karten)}

VAR   I:           WORD;  {Laufvariable}
      INHLSR:      BYTE;  {IO-Inhalt LSR-Register}
      INHIIR:      BYTE;  {IO-Inhalt IIR-Register}

BEGIN
  Writeln;
  Writeln(' COMFIND V',Version,' Suche nach "moeglichen" COM-Portadressen beginnt:');
  Writeln;
  FOR I:=IOMIN TO IOMAX DO
    BEGIN
      INHLSR:=Port[I];
      {1. Suchbedingung: Line-Status-Reg. muss Bitmuster $60H enthalten.}
      IF ((INHLSR AND LSRMSK)=LSRMSK) AND (INHLSR<>$FF) THEN
        BEGIN
         INHIIR:=Port[I-IIROFF];
         {2. Suchbedingung: Interrupt Identification Registerwerte}
         IF INHIIR IN        {Folgende IIR-Registerwerte sind moeglich}
                      [$00,  {8250/16450 MODEM-Status-Interrupt}
                       $01,  {8250/16450 Es liegt kein Interrupt vor}
                       $02,  {8250/16450 Transmitter Holding Reg. Empty Int.}
                       $04,  {8250/16450 Empfangsdateninterrupt}
                       $c0,  {16550 MODEM-Status-Interrupt}
                       $c1,  {16550 Es liegt kein Interrupt vor}
                       $c2,  {16550 Transmitter Holding Reg. Empty Int.}
                       $c4,  {16550 Empfangsdateninterrupt}
                       $cc]  {16550 FIFO Time Out Interrupt}
                       THEN
          BEGIN
           {3. Bedingung: Nur sinnvolle Basisadressen betrachten}
           {              Basisadresse kann nur "0" ooder "8" sein}
           IF ((I-LSROFF) AND $000F) IN [0,8] THEN
             BEGIN
               WRITE('Basisadresse COM-Port HEX: ',HEXWORD(I-LSROFF));
               IF (I-LSROFF)=$3F8 THEN WRITE ('  ==> COM1') ELSE
               IF (I-LSROFF)=$2F8 THEN WRITE ('  ==> COM2') ELSE
               IF (I-LSROFF)=$3E8 THEN WRITE ('  ==> COM3') ELSE
               IF (I-LSROFF)=$2E8 THEN WRITE ('  ==> COM4') ELSE
                                       WRITE ('          ');
               IF (Port[I-LSROff+MCROff] AND Out2)=Out2 THEN
                 WRITELN('  IRQ-Ausgang ist durchgeschaltet!')
               ELSE
                 WRITELN('  IRQ-Ausgang ist inaktiv (Tri-State)!');
               Write('                                           ');
               IF (PORT[I-IIROff] AND $C0)=$C0
                 THEN Writeln('UART=16550. FIFOs sind aktiviert.')
               ELSE
                 BEGIN
                   Port[I-FCROff]:=FIFOON;
                   IF (PORT[I-IIROff] AND $C0)=$C0 THEN 
                     Writeln('UART=16550. FIFOs wurden aktiviert.')
                   ELSE Writeln('UART=8250/16450.');
                 END;
           END;
          END;
        END;
    END;
END.
